@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,32 +15,49 @@
 #
 
 from .services.admin_service import AdminServiceClient
+from .services.admin_service import AdminServiceAsyncClient
 from .services.cursor_service import CursorServiceClient
+from .services.cursor_service import CursorServiceAsyncClient
 from .services.partition_assignment_service import PartitionAssignmentServiceClient
+from .services.partition_assignment_service import PartitionAssignmentServiceAsyncClient
 from .services.publisher_service import PublisherServiceClient
+from .services.publisher_service import PublisherServiceAsyncClient
 from .services.subscriber_service import SubscriberServiceClient
+from .services.subscriber_service import SubscriberServiceAsyncClient
 from .services.topic_stats_service import TopicStatsServiceClient
+from .services.topic_stats_service import TopicStatsServiceAsyncClient
+
+from .types.admin import CreateReservationRequest
 from .types.admin import CreateSubscriptionRequest
 from .types.admin import CreateTopicRequest
+from .types.admin import DeleteReservationRequest
 from .types.admin import DeleteSubscriptionRequest
 from .types.admin import DeleteTopicRequest
+from .types.admin import GetReservationRequest
 from .types.admin import GetSubscriptionRequest
 from .types.admin import GetTopicPartitionsRequest
 from .types.admin import GetTopicRequest
+from .types.admin import ListReservationsRequest
+from .types.admin import ListReservationsResponse
+from .types.admin import ListReservationTopicsRequest
+from .types.admin import ListReservationTopicsResponse
 from .types.admin import ListSubscriptionsRequest
 from .types.admin import ListSubscriptionsResponse
-from .types.admin import ListTopicSubscriptionsRequest
-from .types.admin import ListTopicSubscriptionsResponse
 from .types.admin import ListTopicsRequest
 from .types.admin import ListTopicsResponse
+from .types.admin import ListTopicSubscriptionsRequest
+from .types.admin import ListTopicSubscriptionsResponse
 from .types.admin import TopicPartitions
+from .types.admin import UpdateReservationRequest
 from .types.admin import UpdateSubscriptionRequest
 from .types.admin import UpdateTopicRequest
 from .types.common import AttributeValues
 from .types.common import Cursor
 from .types.common import PubSubMessage
+from .types.common import Reservation
 from .types.common import SequencedMessage
 from .types.common import Subscription
+from .types.common import TimeTarget
 from .types.common import Topic
 from .types.cursor import CommitCursorRequest
 from .types.cursor import CommitCursorResponse
@@ -76,9 +92,17 @@ from .types.topic_stats import ComputeHeadCursorRequest
 from .types.topic_stats import ComputeHeadCursorResponse
 from .types.topic_stats import ComputeMessageStatsRequest
 from .types.topic_stats import ComputeMessageStatsResponse
-
+from .types.topic_stats import ComputeTimeCursorRequest
+from .types.topic_stats import ComputeTimeCursorResponse
 
 __all__ = (
+    "AdminServiceAsyncClient",
+    "CursorServiceAsyncClient",
+    "PartitionAssignmentServiceAsyncClient",
+    "PublisherServiceAsyncClient",
+    "SubscriberServiceAsyncClient",
+    "TopicStatsServiceAsyncClient",
+    "AdminServiceClient",
     "AttributeValues",
     "CommitCursorRequest",
     "CommitCursorResponse",
@@ -86,13 +110,18 @@ __all__ = (
     "ComputeHeadCursorResponse",
     "ComputeMessageStatsRequest",
     "ComputeMessageStatsResponse",
+    "ComputeTimeCursorRequest",
+    "ComputeTimeCursorResponse",
+    "CreateReservationRequest",
     "CreateSubscriptionRequest",
     "CreateTopicRequest",
     "Cursor",
     "CursorServiceClient",
+    "DeleteReservationRequest",
     "DeleteSubscriptionRequest",
     "DeleteTopicRequest",
     "FlowControlRequest",
+    "GetReservationRequest",
     "GetSubscriptionRequest",
     "GetTopicPartitionsRequest",
     "GetTopicRequest",
@@ -105,6 +134,10 @@ __all__ = (
     "InitialSubscribeResponse",
     "ListPartitionCursorsRequest",
     "ListPartitionCursorsResponse",
+    "ListReservationTopicsRequest",
+    "ListReservationTopicsResponse",
+    "ListReservationsRequest",
+    "ListReservationsResponse",
     "ListSubscriptionsRequest",
     "ListSubscriptionsResponse",
     "ListTopicSubscriptionsRequest",
@@ -123,6 +156,7 @@ __all__ = (
     "PublishRequest",
     "PublishResponse",
     "PublisherServiceClient",
+    "Reservation",
     "SeekRequest",
     "SeekResponse",
     "SequencedCommitCursorRequest",
@@ -134,10 +168,11 @@ __all__ = (
     "SubscribeResponse",
     "SubscriberServiceClient",
     "Subscription",
+    "TimeTarget",
     "Topic",
     "TopicPartitions",
     "TopicStatsServiceClient",
+    "UpdateReservationRequest",
     "UpdateSubscriptionRequest",
     "UpdateTopicRequest",
-    "AdminServiceClient",
 )
