@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,22 +13,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import warnings
-from typing import Callable, Dict, Optional, Sequence, Tuple
+from typing import Callable, Dict, Optional, Sequence, Tuple, Union
 
 from google.api_core import grpc_helpers  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
-from google import auth  # type: ignore
-from google.auth import credentials  # type: ignore
+import google.auth  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 
 import grpc  # type: ignore
 
 from google.cloud.pubsublite_v1.types import admin
 from google.cloud.pubsublite_v1.types import common
-from google.protobuf import empty_pb2 as empty  # type: ignore
-
+from google.protobuf import empty_pb2  # type: ignore
 from .base import AdminServiceTransport, DEFAULT_CLIENT_INFO
 
 
@@ -54,7 +51,7 @@ class AdminServiceGrpcTransport(AdminServiceTransport):
         self,
         *,
         host: str = "pubsublite.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: str = None,
         scopes: Sequence[str] = None,
         channel: grpc.Channel = None,
@@ -68,7 +65,8 @@ class AdminServiceGrpcTransport(AdminServiceTransport):
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -110,7 +108,9 @@ class AdminServiceGrpcTransport(AdminServiceTransport):
           google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
               and ``credentials_file`` are passed.
         """
+        self._grpc_channel = None
         self._ssl_channel_credentials = ssl_channel_credentials
+        self._stubs: Dict[str, Callable] = {}
 
         if api_mtls_endpoint:
             warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
@@ -118,94 +118,65 @@ class AdminServiceGrpcTransport(AdminServiceTransport):
             warnings.warn("client_cert_source is deprecated", DeprecationWarning)
 
         if channel:
-            # Sanity check: Ensure that channel and credentials are not both
-            # provided.
+            # Ignore credentials if a channel was passed.
             credentials = False
-
             # If a channel was explicitly provided, set it.
             self._grpc_channel = channel
             self._ssl_channel_credentials = None
-        elif api_mtls_endpoint:
-            host = (
-                api_mtls_endpoint
-                if ":" in api_mtls_endpoint
-                else api_mtls_endpoint + ":443"
-            )
 
-            if credentials is None:
-                credentials, _ = auth.default(
-                    scopes=self.AUTH_SCOPES, quota_project_id=quota_project_id
-                )
-
-            # Create SSL credentials with client_cert_source or application
-            # default SSL credentials.
-            if client_cert_source:
-                cert, key = client_cert_source()
-                ssl_credentials = grpc.ssl_channel_credentials(
-                    certificate_chain=cert, private_key=key
-                )
-            else:
-                ssl_credentials = SslCredentials().ssl_credentials
-
-            # create a new channel. The provided one is ignored.
-            self._grpc_channel = type(self).create_channel(
-                host,
-                credentials=credentials,
-                credentials_file=credentials_file,
-                ssl_credentials=ssl_credentials,
-                scopes=scopes or self.AUTH_SCOPES,
-                quota_project_id=quota_project_id,
-                options=[
-                    ("grpc.max_send_message_length", -1),
-                    ("grpc.max_receive_message_length", -1),
-                ],
-            )
-            self._ssl_channel_credentials = ssl_credentials
         else:
-            host = host if ":" in host else host + ":443"
+            if api_mtls_endpoint:
+                host = api_mtls_endpoint
 
-            if credentials is None:
-                credentials, _ = auth.default(
-                    scopes=self.AUTH_SCOPES, quota_project_id=quota_project_id
-                )
+                # Create SSL credentials with client_cert_source or application
+                # default SSL credentials.
+                if client_cert_source:
+                    cert, key = client_cert_source()
+                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
+                        certificate_chain=cert, private_key=key
+                    )
+                else:
+                    self._ssl_channel_credentials = SslCredentials().ssl_credentials
 
-            if client_cert_source_for_mtls and not ssl_channel_credentials:
-                cert, key = client_cert_source_for_mtls()
-                self._ssl_channel_credentials = grpc.ssl_channel_credentials(
-                    certificate_chain=cert, private_key=key
-                )
+            else:
+                if client_cert_source_for_mtls and not ssl_channel_credentials:
+                    cert, key = client_cert_source_for_mtls()
+                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
+                        certificate_chain=cert, private_key=key
+                    )
 
-            # create a new channel. The provided one is ignored.
-            self._grpc_channel = type(self).create_channel(
-                host,
-                credentials=credentials,
-                credentials_file=credentials_file,
-                ssl_credentials=self._ssl_channel_credentials,
-                scopes=scopes or self.AUTH_SCOPES,
-                quota_project_id=quota_project_id,
-                options=[
-                    ("grpc.max_send_message_length", -1),
-                    ("grpc.max_receive_message_length", -1),
-                ],
-            )
-
-        self._stubs = {}  # type: Dict[str, Callable]
-
-        # Run the base constructor.
+        # The base transport sets the host, credentials and scopes
         super().__init__(
             host=host,
             credentials=credentials,
             credentials_file=credentials_file,
-            scopes=scopes or self.AUTH_SCOPES,
+            scopes=scopes,
             quota_project_id=quota_project_id,
             client_info=client_info,
         )
 
+        if not self._grpc_channel:
+            self._grpc_channel = type(self).create_channel(
+                self._host,
+                credentials=self._credentials,
+                credentials_file=credentials_file,
+                scopes=self._scopes,
+                ssl_credentials=self._ssl_channel_credentials,
+                quota_project_id=quota_project_id,
+                options=[
+                    ("grpc.max_send_message_length", -1),
+                    ("grpc.max_receive_message_length", -1),
+                ],
+            )
+
+        # Wrap messages. This must be done after self._grpc_channel exists
+        self._prep_wrapped_messages(client_info)
+
     @classmethod
     def create_channel(
         cls,
         host: str = "pubsublite.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: str = None,
         scopes: Optional[Sequence[str]] = None,
         quota_project_id: Optional[str] = None,
@@ -213,7 +184,7 @@ class AdminServiceGrpcTransport(AdminServiceTransport):
     ) -> grpc.Channel:
         """Create and return a gRPC channel object.
         Args:
-            address (Optional[str]): The host for the channel to use.
+            host (Optional[str]): The host for the channel to use.
             credentials (Optional[~.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify this application to the service. If
@@ -236,13 +207,15 @@ class AdminServiceGrpcTransport(AdminServiceTransport):
             google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
               and ``credentials_file`` are passed.
         """
-        scopes = scopes or cls.AUTH_SCOPES
+
+        self_signed_jwt_kwargs = cls._get_self_signed_jwt_kwargs(host, scopes)
+
         return grpc_helpers.create_channel(
             host,
             credentials=credentials,
             credentials_file=credentials_file,
-            scopes=scopes,
             quota_project_id=quota_project_id,
+            **self_signed_jwt_kwargs,
             **kwargs,
         )
 
@@ -378,7 +351,7 @@ class AdminServiceGrpcTransport(AdminServiceTransport):
         return self._stubs["update_topic"]
 
     @property
-    def delete_topic(self) -> Callable[[admin.DeleteTopicRequest], empty.Empty]:
+    def delete_topic(self) -> Callable[[admin.DeleteTopicRequest], empty_pb2.Empty]:
         r"""Return a callable for the delete topic method over gRPC.
 
         Deletes the specified topic.
@@ -397,7 +370,7 @@ class AdminServiceGrpcTransport(AdminServiceTransport):
             self._stubs["delete_topic"] = self.grpc_channel.unary_unary(
                 "/google.cloud.pubsublite.v1.AdminService/DeleteTopic",
                 request_serializer=admin.DeleteTopicRequest.serialize,
-                response_deserializer=empty.Empty.FromString,
+                response_deserializer=empty_pb2.Empty.FromString,
             )
         return self._stubs["delete_topic"]
 
@@ -538,7 +511,7 @@ class AdminServiceGrpcTransport(AdminServiceTransport):
     @property
     def delete_subscription(
         self,
-    ) -> Callable[[admin.DeleteSubscriptionRequest], empty.Empty]:
+    ) -> Callable[[admin.DeleteSubscriptionRequest], empty_pb2.Empty]:
         r"""Return a callable for the delete subscription method over gRPC.
 
         Deletes the specified subscription.
@@ -557,9 +530,169 @@ class AdminServiceGrpcTransport(AdminServiceTransport):
             self._stubs["delete_subscription"] = self.grpc_channel.unary_unary(
                 "/google.cloud.pubsublite.v1.AdminService/DeleteSubscription",
                 request_serializer=admin.DeleteSubscriptionRequest.serialize,
-                response_deserializer=empty.Empty.FromString,
+                response_deserializer=empty_pb2.Empty.FromString,
             )
         return self._stubs["delete_subscription"]
 
+    @property
+    def create_reservation(
+        self,
+    ) -> Callable[[admin.CreateReservationRequest], common.Reservation]:
+        r"""Return a callable for the create reservation method over gRPC.
+
+        Creates a new reservation.
+
+        Returns:
+            Callable[[~.CreateReservationRequest],
+                    ~.Reservation]:
+                A function that, when called, will call the underlying RPC
+                on the server.
+        """
+        # Generate a "stub function" on-the-fly which will actually make
+        # the request.
+        # gRPC handles serialization and deserialization, so we just need
+        # to pass in the functions for each.
+        if "create_reservation" not in self._stubs:
+            self._stubs["create_reservation"] = self.grpc_channel.unary_unary(
+                "/google.cloud.pubsublite.v1.AdminService/CreateReservation",
+                request_serializer=admin.CreateReservationRequest.serialize,
+                response_deserializer=common.Reservation.deserialize,
+            )
+        return self._stubs["create_reservation"]
+
+    @property
+    def get_reservation(
+        self,
+    ) -> Callable[[admin.GetReservationRequest], common.Reservation]:
+        r"""Return a callable for the get reservation method over gRPC.
+
+        Returns the reservation configuration.
+
+        Returns:
+            Callable[[~.GetReservationRequest],
+                    ~.Reservation]:
+                A function that, when called, will call the underlying RPC
+                on the server.
+        """
+        # Generate a "stub function" on-the-fly which will actually make
+        # the request.
+        # gRPC handles serialization and deserialization, so we just need
+        # to pass in the functions for each.
+        if "get_reservation" not in self._stubs:
+            self._stubs["get_reservation"] = self.grpc_channel.unary_unary(
+                "/google.cloud.pubsublite.v1.AdminService/GetReservation",
+                request_serializer=admin.GetReservationRequest.serialize,
+                response_deserializer=common.Reservation.deserialize,
+            )
+        return self._stubs["get_reservation"]
+
+    @property
+    def list_reservations(
+        self,
+    ) -> Callable[[admin.ListReservationsRequest], admin.ListReservationsResponse]:
+        r"""Return a callable for the list reservations method over gRPC.
+
+        Returns the list of reservations for the given
+        project.
+
+        Returns:
+            Callable[[~.ListReservationsRequest],
+                    ~.ListReservationsResponse]:
+                A function that, when called, will call the underlying RPC
+                on the server.
+        """
+        # Generate a "stub function" on-the-fly which will actually make
+        # the request.
+        # gRPC handles serialization and deserialization, so we just need
+        # to pass in the functions for each.
+        if "list_reservations" not in self._stubs:
+            self._stubs["list_reservations"] = self.grpc_channel.unary_unary(
+                "/google.cloud.pubsublite.v1.AdminService/ListReservations",
+                request_serializer=admin.ListReservationsRequest.serialize,
+                response_deserializer=admin.ListReservationsResponse.deserialize,
+            )
+        return self._stubs["list_reservations"]
+
+    @property
+    def update_reservation(
+        self,
+    ) -> Callable[[admin.UpdateReservationRequest], common.Reservation]:
+        r"""Return a callable for the update reservation method over gRPC.
+
+        Updates properties of the specified reservation.
+
+        Returns:
+            Callable[[~.UpdateReservationRequest],
+                    ~.Reservation]:
+                A function that, when called, will call the underlying RPC
+                on the server.
+        """
+        # Generate a "stub function" on-the-fly which will actually make
+        # the request.
+        # gRPC handles serialization and deserialization, so we just need
+        # to pass in the functions for each.
+        if "update_reservation" not in self._stubs:
+            self._stubs["update_reservation"] = self.grpc_channel.unary_unary(
+                "/google.cloud.pubsublite.v1.AdminService/UpdateReservation",
+                request_serializer=admin.UpdateReservationRequest.serialize,
+                response_deserializer=common.Reservation.deserialize,
+            )
+        return self._stubs["update_reservation"]
+
+    @property
+    def delete_reservation(
+        self,
+    ) -> Callable[[admin.DeleteReservationRequest], empty_pb2.Empty]:
+        r"""Return a callable for the delete reservation method over gRPC.
+
+        Deletes the specified reservation.
+
+        Returns:
+            Callable[[~.DeleteReservationRequest],
+                    ~.Empty]:
+                A function that, when called, will call the underlying RPC
+                on the server.
+        """
+        # Generate a "stub function" on-the-fly which will actually make
+        # the request.
+        # gRPC handles serialization and deserialization, so we just need
+        # to pass in the functions for each.
+        if "delete_reservation" not in self._stubs:
+            self._stubs["delete_reservation"] = self.grpc_channel.unary_unary(
+                "/google.cloud.pubsublite.v1.AdminService/DeleteReservation",
+                request_serializer=admin.DeleteReservationRequest.serialize,
+                response_deserializer=empty_pb2.Empty.FromString,
+            )
+        return self._stubs["delete_reservation"]
+
+    @property
+    def list_reservation_topics(
+        self,
+    ) -> Callable[
+        [admin.ListReservationTopicsRequest], admin.ListReservationTopicsResponse
+    ]:
+        r"""Return a callable for the list reservation topics method over gRPC.
+
+        Lists the topics attached to the specified
+        reservation.
+
+        Returns:
+            Callable[[~.ListReservationTopicsRequest],
+                    ~.ListReservationTopicsResponse]:
+                A function that, when called, will call the underlying RPC
+                on the server.
+        """
+        # Generate a "stub function" on-the-fly which will actually make
+        # the request.
+        # gRPC handles serialization and deserialization, so we just need
+        # to pass in the functions for each.
+        if "list_reservation_topics" not in self._stubs:
+            self._stubs["list_reservation_topics"] = self.grpc_channel.unary_unary(
+                "/google.cloud.pubsublite.v1.AdminService/ListReservationTopics",
+                request_serializer=admin.ListReservationTopicsRequest.serialize,
+                response_deserializer=admin.ListReservationTopicsResponse.deserialize,
+            )
+        return self._stubs["list_reservation_topics"]
+
 
 __all__ = ("AdminServiceGrpcTransport",)
