@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,7 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 from collections import OrderedDict
 from distutils import util
 import os
@@ -33,10 +31,10 @@ from typing import (
 import pkg_resources
 
 from google.api_core import client_options as client_options_lib  # type: ignore
-from google.api_core import exceptions  # type: ignore
+from google.api_core import exceptions as core_exceptions  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
-from google.auth import credentials  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.auth.exceptions import MutualTLSChannelError  # type: ignore
@@ -44,7 +42,6 @@ from google.oauth2 import service_account  # type: ignore
 
 from google.cloud.pubsublite_v1.services.cursor_service import pagers
 from google.cloud.pubsublite_v1.types import cursor
-
 from .transports.base import CursorServiceTransport, DEFAULT_CLIENT_INFO
 from .transports.grpc import CursorServiceGrpcTransport
 from .transports.grpc_asyncio import CursorServiceGrpcAsyncIOTransport
@@ -63,7 +60,7 @@ class CursorServiceClientMeta(type):
     _transport_registry["grpc_asyncio"] = CursorServiceGrpcAsyncIOTransport
 
     def get_transport_class(cls, label: str = None,) -> Type[CursorServiceTransport]:
-        """Return an appropriate transport class.
+        """Returns an appropriate transport class.
 
         Args:
             label: The name of the desired transport. If none is
@@ -90,7 +87,8 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
 
     @staticmethod
     def _get_default_mtls_endpoint(api_endpoint):
-        """Convert api endpoint to mTLS endpoint.
+        """Converts api endpoint to mTLS endpoint.
+
         Convert "*.sandbox.googleapis.com" and "*.googleapis.com" to
         "*.mtls.sandbox.googleapis.com" and "*.mtls.googleapis.com" respectively.
         Args:
@@ -122,10 +120,27 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
         DEFAULT_ENDPOINT
     )
 
+    @classmethod
+    def from_service_account_info(cls, info: dict, *args, **kwargs):
+        """Creates an instance of this client using the provided credentials
+            info.
+
+        Args:
+            info (dict): The service account private key info.
+            args: Additional arguments to pass to the constructor.
+            kwargs: Additional arguments to pass to the constructor.
+
+        Returns:
+            CursorServiceClient: The constructed client.
+        """
+        credentials = service_account.Credentials.from_service_account_info(info)
+        kwargs["credentials"] = credentials
+        return cls(*args, **kwargs)
+
     @classmethod
     def from_service_account_file(cls, filename: str, *args, **kwargs):
         """Creates an instance of this client using the provided credentials
-        file.
+            file.
 
         Args:
             filename (str): The path to the service account private key json
@@ -134,7 +149,7 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
             kwargs: Additional arguments to pass to the constructor.
 
         Returns:
-            {@api.name}: The constructed client.
+            CursorServiceClient: The constructed client.
         """
         credentials = service_account.Credentials.from_service_account_file(filename)
         kwargs["credentials"] = credentials
@@ -144,23 +159,24 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
 
     @property
     def transport(self) -> CursorServiceTransport:
-        """Return the transport used by the client instance.
+        """Returns the transport used by the client instance.
 
         Returns:
-            CursorServiceTransport: The transport used by the client instance.
+            CursorServiceTransport: The transport used by the client
+                instance.
         """
         return self._transport
 
     @staticmethod
     def subscription_path(project: str, location: str, subscription: str,) -> str:
-        """Return a fully-qualified subscription string."""
+        """Returns a fully-qualified subscription string."""
         return "projects/{project}/locations/{location}/subscriptions/{subscription}".format(
             project=project, location=location, subscription=subscription,
         )
 
     @staticmethod
     def parse_subscription_path(path: str) -> Dict[str, str]:
-        """Parse a subscription path into its component segments."""
+        """Parses a subscription path into its component segments."""
         m = re.match(
             r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)/subscriptions/(?P<subscription>.+?)$",
             path,
@@ -169,7 +185,7 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
 
     @staticmethod
     def common_billing_account_path(billing_account: str,) -> str:
-        """Return a fully-qualified billing_account string."""
+        """Returns a fully-qualified billing_account string."""
         return "billingAccounts/{billing_account}".format(
             billing_account=billing_account,
         )
@@ -182,7 +198,7 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
 
     @staticmethod
     def common_folder_path(folder: str,) -> str:
-        """Return a fully-qualified folder string."""
+        """Returns a fully-qualified folder string."""
         return "folders/{folder}".format(folder=folder,)
 
     @staticmethod
@@ -193,7 +209,7 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
 
     @staticmethod
     def common_organization_path(organization: str,) -> str:
-        """Return a fully-qualified organization string."""
+        """Returns a fully-qualified organization string."""
         return "organizations/{organization}".format(organization=organization,)
 
     @staticmethod
@@ -204,7 +220,7 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
 
     @staticmethod
     def common_project_path(project: str,) -> str:
-        """Return a fully-qualified project string."""
+        """Returns a fully-qualified project string."""
         return "projects/{project}".format(project=project,)
 
     @staticmethod
@@ -215,7 +231,7 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
 
     @staticmethod
     def common_location_path(project: str, location: str,) -> str:
-        """Return a fully-qualified location string."""
+        """Returns a fully-qualified location string."""
         return "projects/{project}/locations/{location}".format(
             project=project, location=location,
         )
@@ -229,12 +245,12 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
     def __init__(
         self,
         *,
-        credentials: Optional[credentials.Credentials] = None,
+        credentials: Optional[ga_credentials.Credentials] = None,
         transport: Union[str, CursorServiceTransport, None] = None,
         client_options: Optional[client_options_lib.ClientOptions] = None,
         client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
     ) -> None:
-        """Instantiate the cursor service client.
+        """Instantiates the cursor service client.
 
         Args:
             credentials (Optional[google.auth.credentials.Credentials]): The
@@ -242,10 +258,10 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
                 credentials identify the application to the service; if none
                 are specified, the client will attempt to ascertain the
                 credentials from the environment.
-            transport (Union[str, ~.CursorServiceTransport]): The
+            transport (Union[str, CursorServiceTransport]): The
                 transport to use. If set to None, a transport is chosen
                 automatically.
-            client_options (client_options_lib.ClientOptions): Custom options for the
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                 client. It won't take effect if a ``transport`` instance is provided.
                 (1) The ``api_endpoint`` property can be used to override the
                 default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
@@ -281,21 +297,18 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
             util.strtobool(os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"))
         )
 
-        ssl_credentials = None
+        client_cert_source_func = None
         is_mtls = False
         if use_client_cert:
             if client_options.client_cert_source:
-                import grpc  # type: ignore
-
-                cert, key = client_options.client_cert_source()
-                ssl_credentials = grpc.ssl_channel_credentials(
-                    certificate_chain=cert, private_key=key
-                )
                 is_mtls = True
+                client_cert_source_func = client_options.client_cert_source
             else:
-                creds = SslCredentials()
-                is_mtls = creds.is_mtls
-                ssl_credentials = creds.ssl_credentials if is_mtls else None
+                is_mtls = mtls.has_default_client_cert_source()
+                if is_mtls:
+                    client_cert_source_func = mtls.default_client_cert_source()
+                else:
+                    client_cert_source_func = None
 
         # Figure out which api endpoint to use.
         if client_options.api_endpoint is not None:
@@ -307,12 +320,14 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
             elif use_mtls_env == "always":
                 api_endpoint = self.DEFAULT_MTLS_ENDPOINT
             elif use_mtls_env == "auto":
-                api_endpoint = (
-                    self.DEFAULT_MTLS_ENDPOINT if is_mtls else self.DEFAULT_ENDPOINT
-                )
+                if is_mtls:
+                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
+                else:
+                    api_endpoint = self.DEFAULT_ENDPOINT
             else:
                 raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted values: never, auto, always"
+                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
+                    "values: never, auto, always"
                 )
 
         # Save or instantiate the transport.
@@ -327,8 +342,8 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
                 )
             if client_options.scopes:
                 raise ValueError(
-                    "When providing a transport instance, "
-                    "provide its scopes directly."
+                    "When providing a transport instance, provide its scopes "
+                    "directly."
                 )
             self._transport = transport
         else:
@@ -338,7 +353,7 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
                 credentials_file=client_options.credentials_file,
                 host=api_endpoint,
                 scopes=client_options.scopes,
-                ssl_channel_credentials=ssl_credentials,
+                client_cert_source_for_mtls=client_cert_source_func,
                 quota_project_id=client_options.quota_project_id,
                 client_info=client_info,
             )
@@ -355,7 +370,7 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
         committed cursors.
 
         Args:
-            requests (Iterator[`~.cursor.StreamingCommitCursorRequest`]):
+            requests (Iterator[google.cloud.pubsublite_v1.types.StreamingCommitCursorRequest]):
                 The request object iterator. A request sent from the client to
                 the server on a stream.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -365,7 +380,7 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
                 sent along with the request as metadata.
 
         Returns:
-            Iterable[~.cursor.StreamingCommitCursorResponse]:
+            Iterable[google.cloud.pubsublite_v1.types.StreamingCommitCursorResponse]:
                 Response to a
                 StreamingCommitCursorRequest.
 
@@ -392,9 +407,8 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
         r"""Updates the committed cursor.
 
         Args:
-            request (:class:`~.cursor.CommitCursorRequest`):
+            request (google.cloud.pubsublite_v1.types.CommitCursorRequest):
                 The request object. Request for CommitCursor.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -402,11 +416,10 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
                 sent along with the request as metadata.
 
         Returns:
-            ~.cursor.CommitCursorResponse:
+            google.cloud.pubsublite_v1.types.CommitCursorResponse:
                 Response for CommitCursor.
         """
         # Create or coerce a protobuf request object.
-
         # Minor optimization to avoid making a copy if the user passes
         # in a cursor.CommitCursorRequest.
         # There's no risk of modifying the input as we've already verified
@@ -418,6 +431,14 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
         # and friendly error handling.
         rpc = self._transport._wrapped_methods[self._transport.commit_cursor]
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata(
+                (("subscription", request.subscription),)
+            ),
+        )
+
         # Send the request.
         response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
 
@@ -437,16 +458,16 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
         subscription.
 
         Args:
-            request (:class:`~.cursor.ListPartitionCursorsRequest`):
+            request (google.cloud.pubsublite_v1.types.ListPartitionCursorsRequest):
                 The request object. Request for ListPartitionCursors.
-            parent (:class:`str`):
+            parent (str):
                 Required. The subscription for which to retrieve
                 cursors. Structured like
                 ``projects/{project_number}/locations/{location}/subscriptions/{subscription_id}``.
+
                 This corresponds to the ``parent`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
             timeout (float): The timeout for this request.
@@ -454,7 +475,7 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
                 sent along with the request as metadata.
 
         Returns:
-            ~.pagers.ListPartitionCursorsPager:
+            google.cloud.pubsublite_v1.services.cursor_service.pagers.ListPartitionCursorsPager:
                 Response for ListPartitionCursors
                 Iterating over this object will yield
                 results and resolve additional pages
@@ -477,10 +498,8 @@ class CursorServiceClient(metaclass=CursorServiceClientMeta):
         # there are no flattened fields.
         if not isinstance(request, cursor.ListPartitionCursorsRequest):
             request = cursor.ListPartitionCursorsRequest(request)
-
             # If we have keyword arguments corresponding to fields on the
             # request, apply these.
-
             if parent is not None:
                 request.parent = parent
 
