@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,9 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import os
 import mock
+import packaging.version
 
 import grpc
 from grpc.experimental import aio
@@ -24,21 +23,51 @@ import math
 import pytest
 from proto.marshal.rules.dates import DurationRule, TimestampRule
 
-from google import auth
+
 from google.api_core import client_options
-from google.api_core import exceptions
+from google.api_core import exceptions as core_exceptions
 from google.api_core import gapic_v1
 from google.api_core import grpc_helpers
 from google.api_core import grpc_helpers_async
-from google.auth import credentials
+from google.auth import credentials as ga_credentials
 from google.auth.exceptions import MutualTLSChannelError
 from google.cloud.pubsublite_v1.services.cursor_service import CursorServiceAsyncClient
 from google.cloud.pubsublite_v1.services.cursor_service import CursorServiceClient
 from google.cloud.pubsublite_v1.services.cursor_service import pagers
 from google.cloud.pubsublite_v1.services.cursor_service import transports
+from google.cloud.pubsublite_v1.services.cursor_service.transports.base import (
+    _API_CORE_VERSION,
+)
+from google.cloud.pubsublite_v1.services.cursor_service.transports.base import (
+    _GOOGLE_AUTH_VERSION,
+)
 from google.cloud.pubsublite_v1.types import common
 from google.cloud.pubsublite_v1.types import cursor
 from google.oauth2 import service_account
+import google.auth
+
+
+# TODO(busunkim): Once google-api-core >= 1.26.0 is required:
+# - Delete all the api-core and auth "less than" test cases
+# - Delete these pytest markers (Make the "greater than or equal to" tests the default).
+requires_google_auth_lt_1_25_0 = pytest.mark.skipif(
+    packaging.version.parse(_GOOGLE_AUTH_VERSION) >= packaging.version.parse("1.25.0"),
+    reason="This test requires google-auth < 1.25.0",
+)
+requires_google_auth_gte_1_25_0 = pytest.mark.skipif(
+    packaging.version.parse(_GOOGLE_AUTH_VERSION) < packaging.version.parse("1.25.0"),
+    reason="This test requires google-auth >= 1.25.0",
+)
+
+requires_api_core_lt_1_26_0 = pytest.mark.skipif(
+    packaging.version.parse(_API_CORE_VERSION) >= packaging.version.parse("1.26.0"),
+    reason="This test requires google-api-core < 1.26.0",
+)
+
+requires_api_core_gte_1_26_0 = pytest.mark.skipif(
+    packaging.version.parse(_API_CORE_VERSION) < packaging.version.parse("1.26.0"),
+    reason="This test requires google-api-core >= 1.26.0",
+)
 
 
 def client_cert_source_callback():
@@ -89,7 +118,7 @@ def test__get_default_mtls_endpoint():
     "client_class", [CursorServiceClient, CursorServiceAsyncClient,]
 )
 def test_cursor_service_client_from_service_account_info(client_class):
-    creds = credentials.AnonymousCredentials()
+    creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_info"
     ) as factory:
@@ -106,7 +135,7 @@ def test_cursor_service_client_from_service_account_info(client_class):
     "client_class", [CursorServiceClient, CursorServiceAsyncClient,]
 )
 def test_cursor_service_client_from_service_account_file(client_class):
-    creds = credentials.AnonymousCredentials()
+    creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_file"
     ) as factory:
@@ -159,7 +188,7 @@ def test_cursor_service_client_client_options(
 ):
     # Check that if channel is provided we won't create a new one.
     with mock.patch.object(CursorServiceClient, "get_transport_class") as gtc:
-        transport = transport_class(credentials=credentials.AnonymousCredentials())
+        transport = transport_class(credentials=ga_credentials.AnonymousCredentials())
         client = client_class(transport=transport)
         gtc.assert_not_called()
 
@@ -447,13 +476,12 @@ def test_streaming_commit_cursor(
     transport: str = "grpc", request_type=cursor.StreamingCommitCursorRequest
 ):
     client = CursorServiceClient(
-        credentials=credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
     # and we are mocking out the actual API, so just send an empty request.
     request = request_type()
-
     requests = [request]
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -462,13 +490,11 @@ def test_streaming_commit_cursor(
     ) as call:
         # Designate an appropriate return value for the call.
         call.return_value = iter([cursor.StreamingCommitCursorResponse()])
-
         response = client.streaming_commit_cursor(iter(requests))
 
         # Establish that the underlying gRPC stub method was called.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-
         assert next(args[0]) == request
 
     # Establish that the response is the type that we expect.
@@ -485,13 +511,12 @@ async def test_streaming_commit_cursor_async(
     transport: str = "grpc_asyncio", request_type=cursor.StreamingCommitCursorRequest
 ):
     client = CursorServiceAsyncClient(
-        credentials=credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
     # and we are mocking out the actual API, so just send an empty request.
     request = request_type()
-
     requests = [request]
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -503,13 +528,11 @@ async def test_streaming_commit_cursor_async(
         call.return_value.read = mock.AsyncMock(
             side_effect=[cursor.StreamingCommitCursorResponse()]
         )
-
         response = await client.streaming_commit_cursor(iter(requests))
 
         # Establish that the underlying gRPC stub method was called.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-
         assert next(args[0]) == request
 
     # Establish that the response is the type that we expect.
@@ -526,7 +549,7 @@ def test_commit_cursor(
     transport: str = "grpc", request_type=cursor.CommitCursorRequest
 ):
     client = CursorServiceClient(
-        credentials=credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -537,17 +560,14 @@ def test_commit_cursor(
     with mock.patch.object(type(client.transport.commit_cursor), "__call__") as call:
         # Designate an appropriate return value for the call.
         call.return_value = cursor.CommitCursorResponse()
-
         response = client.commit_cursor(request)
 
         # Establish that the underlying gRPC stub method was called.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == cursor.CommitCursorRequest()
 
     # Establish that the response is the type that we expect.
-
     assert isinstance(response, cursor.CommitCursorResponse)
 
 
@@ -559,7 +579,7 @@ def test_commit_cursor_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
     client = CursorServiceClient(
-        credentials=credentials.AnonymousCredentials(), transport="grpc",
+        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
     )
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -567,7 +587,6 @@ def test_commit_cursor_empty_call():
         client.commit_cursor()
         call.assert_called()
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == cursor.CommitCursorRequest()
 
 
@@ -576,7 +595,7 @@ async def test_commit_cursor_async(
     transport: str = "grpc_asyncio", request_type=cursor.CommitCursorRequest
 ):
     client = CursorServiceAsyncClient(
-        credentials=credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -589,13 +608,11 @@ async def test_commit_cursor_async(
         call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
             cursor.CommitCursorResponse()
         )
-
         response = await client.commit_cursor(request)
 
         # Establish that the underlying gRPC stub method was called.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == cursor.CommitCursorRequest()
 
     # Establish that the response is the type that we expect.
@@ -607,11 +624,68 @@ async def test_commit_cursor_async_from_dict():
     await test_commit_cursor_async(request_type=dict)
 
 
+def test_commit_cursor_field_headers():
+    client = CursorServiceClient(credentials=ga_credentials.AnonymousCredentials(),)
+
+    # Any value that is part of the HTTP/1.1 URI should be sent as
+    # a field header. Set these to a non-empty value.
+    request = cursor.CommitCursorRequest()
+
+    request.subscription = "subscription/value"
+
+    # Mock the actual call within the gRPC stub, and fake the request.
+    with mock.patch.object(type(client.transport.commit_cursor), "__call__") as call:
+        call.return_value = cursor.CommitCursorResponse()
+        client.commit_cursor(request)
+
+        # Establish that the underlying gRPC stub method was called.
+        assert len(call.mock_calls) == 1
+        _, args, _ = call.mock_calls[0]
+        assert args[0] == request
+
+    # Establish that the field header was sent.
+    _, _, kw = call.mock_calls[0]
+    assert ("x-goog-request-params", "subscription=subscription/value",) in kw[
+        "metadata"
+    ]
+
+
+@pytest.mark.asyncio
+async def test_commit_cursor_field_headers_async():
+    client = CursorServiceAsyncClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+
+    # Any value that is part of the HTTP/1.1 URI should be sent as
+    # a field header. Set these to a non-empty value.
+    request = cursor.CommitCursorRequest()
+
+    request.subscription = "subscription/value"
+
+    # Mock the actual call within the gRPC stub, and fake the request.
+    with mock.patch.object(type(client.transport.commit_cursor), "__call__") as call:
+        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
+            cursor.CommitCursorResponse()
+        )
+        await client.commit_cursor(request)
+
+        # Establish that the underlying gRPC stub method was called.
+        assert len(call.mock_calls)
+        _, args, _ = call.mock_calls[0]
+        assert args[0] == request
+
+    # Establish that the field header was sent.
+    _, _, kw = call.mock_calls[0]
+    assert ("x-goog-request-params", "subscription=subscription/value",) in kw[
+        "metadata"
+    ]
+
+
 def test_list_partition_cursors(
     transport: str = "grpc", request_type=cursor.ListPartitionCursorsRequest
 ):
     client = CursorServiceClient(
-        credentials=credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -626,19 +700,15 @@ def test_list_partition_cursors(
         call.return_value = cursor.ListPartitionCursorsResponse(
             next_page_token="next_page_token_value",
         )
-
         response = client.list_partition_cursors(request)
 
         # Establish that the underlying gRPC stub method was called.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == cursor.ListPartitionCursorsRequest()
 
     # Establish that the response is the type that we expect.
-
     assert isinstance(response, pagers.ListPartitionCursorsPager)
-
     assert response.next_page_token == "next_page_token_value"
 
 
@@ -650,7 +720,7 @@ def test_list_partition_cursors_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
     client = CursorServiceClient(
-        credentials=credentials.AnonymousCredentials(), transport="grpc",
+        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
     )
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -660,7 +730,6 @@ def test_list_partition_cursors_empty_call():
         client.list_partition_cursors()
         call.assert_called()
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == cursor.ListPartitionCursorsRequest()
 
 
@@ -669,7 +738,7 @@ async def test_list_partition_cursors_async(
     transport: str = "grpc_asyncio", request_type=cursor.ListPartitionCursorsRequest
 ):
     client = CursorServiceAsyncClient(
-        credentials=credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -686,18 +755,15 @@ async def test_list_partition_cursors_async(
                 next_page_token="next_page_token_value",
             )
         )
-
         response = await client.list_partition_cursors(request)
 
         # Establish that the underlying gRPC stub method was called.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == cursor.ListPartitionCursorsRequest()
 
     # Establish that the response is the type that we expect.
     assert isinstance(response, pagers.ListPartitionCursorsAsyncPager)
-
     assert response.next_page_token == "next_page_token_value"
 
 
@@ -707,11 +773,12 @@ async def test_list_partition_cursors_async_from_dict():
 
 
 def test_list_partition_cursors_field_headers():
-    client = CursorServiceClient(credentials=credentials.AnonymousCredentials(),)
+    client = CursorServiceClient(credentials=ga_credentials.AnonymousCredentials(),)
 
     # Any value that is part of the HTTP/1.1 URI should be sent as
     # a field header. Set these to a non-empty value.
     request = cursor.ListPartitionCursorsRequest()
+
     request.parent = "parent/value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -719,7 +786,6 @@ def test_list_partition_cursors_field_headers():
         type(client.transport.list_partition_cursors), "__call__"
     ) as call:
         call.return_value = cursor.ListPartitionCursorsResponse()
-
         client.list_partition_cursors(request)
 
         # Establish that the underlying gRPC stub method was called.
@@ -734,11 +800,14 @@ def test_list_partition_cursors_field_headers():
 
 @pytest.mark.asyncio
 async def test_list_partition_cursors_field_headers_async():
-    client = CursorServiceAsyncClient(credentials=credentials.AnonymousCredentials(),)
+    client = CursorServiceAsyncClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
 
     # Any value that is part of the HTTP/1.1 URI should be sent as
     # a field header. Set these to a non-empty value.
     request = cursor.ListPartitionCursorsRequest()
+
     request.parent = "parent/value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -748,7 +817,6 @@ async def test_list_partition_cursors_field_headers_async():
         call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
             cursor.ListPartitionCursorsResponse()
         )
-
         await client.list_partition_cursors(request)
 
         # Establish that the underlying gRPC stub method was called.
@@ -762,7 +830,7 @@ async def test_list_partition_cursors_field_headers_async():
 
 
 def test_list_partition_cursors_flattened():
-    client = CursorServiceClient(credentials=credentials.AnonymousCredentials(),)
+    client = CursorServiceClient(credentials=ga_credentials.AnonymousCredentials(),)
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -770,7 +838,6 @@ def test_list_partition_cursors_flattened():
     ) as call:
         # Designate an appropriate return value for the call.
         call.return_value = cursor.ListPartitionCursorsResponse()
-
         # Call the method with a truthy value for each flattened field,
         # using the keyword arguments to the method.
         client.list_partition_cursors(parent="parent_value",)
@@ -779,12 +846,11 @@ def test_list_partition_cursors_flattened():
         # request object values.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-
         assert args[0].parent == "parent_value"
 
 
 def test_list_partition_cursors_flattened_error():
-    client = CursorServiceClient(credentials=credentials.AnonymousCredentials(),)
+    client = CursorServiceClient(credentials=ga_credentials.AnonymousCredentials(),)
 
     # Attempting to call a method with both a request object and flattened
     # fields is an error.
@@ -796,7 +862,9 @@ def test_list_partition_cursors_flattened_error():
 
 @pytest.mark.asyncio
 async def test_list_partition_cursors_flattened_async():
-    client = CursorServiceAsyncClient(credentials=credentials.AnonymousCredentials(),)
+    client = CursorServiceAsyncClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -816,13 +884,14 @@ async def test_list_partition_cursors_flattened_async():
         # request object values.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-
         assert args[0].parent == "parent_value"
 
 
 @pytest.mark.asyncio
 async def test_list_partition_cursors_flattened_error_async():
-    client = CursorServiceAsyncClient(credentials=credentials.AnonymousCredentials(),)
+    client = CursorServiceAsyncClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
 
     # Attempting to call a method with both a request object and flattened
     # fields is an error.
@@ -833,7 +902,7 @@ async def test_list_partition_cursors_flattened_error_async():
 
 
 def test_list_partition_cursors_pager():
-    client = CursorServiceClient(credentials=credentials.AnonymousCredentials,)
+    client = CursorServiceClient(credentials=ga_credentials.AnonymousCredentials,)
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -875,7 +944,7 @@ def test_list_partition_cursors_pager():
 
 
 def test_list_partition_cursors_pages():
-    client = CursorServiceClient(credentials=credentials.AnonymousCredentials,)
+    client = CursorServiceClient(credentials=ga_credentials.AnonymousCredentials,)
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -909,7 +978,7 @@ def test_list_partition_cursors_pages():
 
 @pytest.mark.asyncio
 async def test_list_partition_cursors_async_pager():
-    client = CursorServiceAsyncClient(credentials=credentials.AnonymousCredentials,)
+    client = CursorServiceAsyncClient(credentials=ga_credentials.AnonymousCredentials,)
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -950,7 +1019,7 @@ async def test_list_partition_cursors_async_pager():
 
 @pytest.mark.asyncio
 async def test_list_partition_cursors_async_pages():
-    client = CursorServiceAsyncClient(credentials=credentials.AnonymousCredentials,)
+    client = CursorServiceAsyncClient(credentials=ga_credentials.AnonymousCredentials,)
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -989,16 +1058,16 @@ async def test_list_partition_cursors_async_pages():
 def test_credentials_transport_error():
     # It is an error to provide credentials and a transport instance.
     transport = transports.CursorServiceGrpcTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     with pytest.raises(ValueError):
         client = CursorServiceClient(
-            credentials=credentials.AnonymousCredentials(), transport=transport,
+            credentials=ga_credentials.AnonymousCredentials(), transport=transport,
         )
 
     # It is an error to provide a credentials file and a transport instance.
     transport = transports.CursorServiceGrpcTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     with pytest.raises(ValueError):
         client = CursorServiceClient(
@@ -1008,7 +1077,7 @@ def test_credentials_transport_error():
 
     # It is an error to provide scopes and a transport instance.
     transport = transports.CursorServiceGrpcTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     with pytest.raises(ValueError):
         client = CursorServiceClient(
@@ -1019,7 +1088,7 @@ def test_credentials_transport_error():
 def test_transport_instance():
     # A client may be instantiated with a custom transport instance.
     transport = transports.CursorServiceGrpcTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     client = CursorServiceClient(transport=transport)
     assert client.transport is transport
@@ -1028,13 +1097,13 @@ def test_transport_instance():
 def test_transport_get_channel():
     # A client may be instantiated with a custom transport instance.
     transport = transports.CursorServiceGrpcTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     channel = transport.grpc_channel
     assert channel
 
     transport = transports.CursorServiceGrpcAsyncIOTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     channel = transport.grpc_channel
     assert channel
@@ -1049,23 +1118,23 @@ def test_transport_get_channel():
 )
 def test_transport_adc(transport_class):
     # Test default credentials are used if not provided.
-    with mock.patch.object(auth, "default") as adc:
-        adc.return_value = (credentials.AnonymousCredentials(), None)
+    with mock.patch.object(google.auth, "default") as adc:
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
         transport_class()
         adc.assert_called_once()
 
 
 def test_transport_grpc_default():
     # A client should use the gRPC transport by default.
-    client = CursorServiceClient(credentials=credentials.AnonymousCredentials(),)
+    client = CursorServiceClient(credentials=ga_credentials.AnonymousCredentials(),)
     assert isinstance(client.transport, transports.CursorServiceGrpcTransport,)
 
 
 def test_cursor_service_base_transport_error():
     # Passing both a credentials object and credentials_file should raise an error
-    with pytest.raises(exceptions.DuplicateCredentialArgs):
+    with pytest.raises(core_exceptions.DuplicateCredentialArgs):
         transport = transports.CursorServiceTransport(
-            credentials=credentials.AnonymousCredentials(),
+            credentials=ga_credentials.AnonymousCredentials(),
             credentials_file="credentials.json",
         )
 
@@ -1077,7 +1146,7 @@ def test_cursor_service_base_transport():
     ) as Transport:
         Transport.return_value = None
         transport = transports.CursorServiceTransport(
-            credentials=credentials.AnonymousCredentials(),
+            credentials=ga_credentials.AnonymousCredentials(),
         )
 
     # Every method on the transport should just blindly
@@ -1092,15 +1161,37 @@ def test_cursor_service_base_transport():
             getattr(transport, method)(request=object())
 
 
+@requires_google_auth_gte_1_25_0
 def test_cursor_service_base_transport_with_credentials_file():
     # Instantiate the base transport with a credentials file
     with mock.patch.object(
-        auth, "load_credentials_from_file"
+        google.auth, "load_credentials_from_file", autospec=True
     ) as load_creds, mock.patch(
         "google.cloud.pubsublite_v1.services.cursor_service.transports.CursorServiceTransport._prep_wrapped_messages"
     ) as Transport:
         Transport.return_value = None
-        load_creds.return_value = (credentials.AnonymousCredentials(), None)
+        load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
+        transport = transports.CursorServiceTransport(
+            credentials_file="credentials.json", quota_project_id="octopus",
+        )
+        load_creds.assert_called_once_with(
+            "credentials.json",
+            scopes=None,
+            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
+            quota_project_id="octopus",
+        )
+
+
+@requires_google_auth_lt_1_25_0
+def test_cursor_service_base_transport_with_credentials_file_old_google_auth():
+    # Instantiate the base transport with a credentials file
+    with mock.patch.object(
+        google.auth, "load_credentials_from_file", autospec=True
+    ) as load_creds, mock.patch(
+        "google.cloud.pubsublite_v1.services.cursor_service.transports.CursorServiceTransport._prep_wrapped_messages"
+    ) as Transport:
+        Transport.return_value = None
+        load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
         transport = transports.CursorServiceTransport(
             credentials_file="credentials.json", quota_project_id="octopus",
         )
@@ -1113,19 +1204,33 @@ def test_cursor_service_base_transport_with_credentials_file():
 
 def test_cursor_service_base_transport_with_adc():
     # Test the default credentials are used if credentials and credentials_file are None.
-    with mock.patch.object(auth, "default") as adc, mock.patch(
+    with mock.patch.object(google.auth, "default", autospec=True) as adc, mock.patch(
         "google.cloud.pubsublite_v1.services.cursor_service.transports.CursorServiceTransport._prep_wrapped_messages"
     ) as Transport:
         Transport.return_value = None
-        adc.return_value = (credentials.AnonymousCredentials(), None)
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
         transport = transports.CursorServiceTransport()
         adc.assert_called_once()
 
 
+@requires_google_auth_gte_1_25_0
 def test_cursor_service_auth_adc():
     # If no credentials are provided, we should use ADC credentials.
-    with mock.patch.object(auth, "default") as adc:
-        adc.return_value = (credentials.AnonymousCredentials(), None)
+    with mock.patch.object(google.auth, "default", autospec=True) as adc:
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
+        CursorServiceClient()
+        adc.assert_called_once_with(
+            scopes=None,
+            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
+            quota_project_id=None,
+        )
+
+
+@requires_google_auth_lt_1_25_0
+def test_cursor_service_auth_adc_old_google_auth():
+    # If no credentials are provided, we should use ADC credentials.
+    with mock.patch.object(google.auth, "default", autospec=True) as adc:
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
         CursorServiceClient()
         adc.assert_called_once_with(
             scopes=("https://www.googleapis.com/auth/cloud-platform",),
@@ -1133,20 +1238,156 @@ def test_cursor_service_auth_adc():
         )
 
 
-def test_cursor_service_transport_auth_adc():
+@pytest.mark.parametrize(
+    "transport_class",
+    [
+        transports.CursorServiceGrpcTransport,
+        transports.CursorServiceGrpcAsyncIOTransport,
+    ],
+)
+@requires_google_auth_gte_1_25_0
+def test_cursor_service_transport_auth_adc(transport_class):
     # If credentials and host are not provided, the transport class should use
     # ADC credentials.
-    with mock.patch.object(auth, "default") as adc:
-        adc.return_value = (credentials.AnonymousCredentials(), None)
-        transports.CursorServiceGrpcTransport(
-            host="squid.clam.whelk", quota_project_id="octopus"
-        )
+    with mock.patch.object(google.auth, "default", autospec=True) as adc:
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
+        transport_class(quota_project_id="octopus", scopes=["1", "2"])
         adc.assert_called_once_with(
-            scopes=("https://www.googleapis.com/auth/cloud-platform",),
+            scopes=["1", "2"],
+            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
             quota_project_id="octopus",
         )
 
 
+@pytest.mark.parametrize(
+    "transport_class",
+    [
+        transports.CursorServiceGrpcTransport,
+        transports.CursorServiceGrpcAsyncIOTransport,
+    ],
+)
+@requires_google_auth_lt_1_25_0
+def test_cursor_service_transport_auth_adc_old_google_auth(transport_class):
+    # If credentials and host are not provided, the transport class should use
+    # ADC credentials.
+    with mock.patch.object(google.auth, "default", autospec=True) as adc:
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
+        transport_class(quota_project_id="octopus")
+        adc.assert_called_once_with(
+            scopes=("https://www.googleapis.com/auth/cloud-platform",),
+            quota_project_id="octopus",
+        )
+
+
+@pytest.mark.parametrize(
+    "transport_class,grpc_helpers",
+    [
+        (transports.CursorServiceGrpcTransport, grpc_helpers),
+        (transports.CursorServiceGrpcAsyncIOTransport, grpc_helpers_async),
+    ],
+)
+@requires_api_core_gte_1_26_0
+def test_cursor_service_transport_create_channel(transport_class, grpc_helpers):
+    # If credentials and host are not provided, the transport class should use
+    # ADC credentials.
+    with mock.patch.object(
+        google.auth, "default", autospec=True
+    ) as adc, mock.patch.object(
+        grpc_helpers, "create_channel", autospec=True
+    ) as create_channel:
+        creds = ga_credentials.AnonymousCredentials()
+        adc.return_value = (creds, None)
+        transport_class(quota_project_id="octopus", scopes=["1", "2"])
+
+        create_channel.assert_called_with(
+            "pubsublite.googleapis.com:443",
+            credentials=creds,
+            credentials_file=None,
+            quota_project_id="octopus",
+            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
+            scopes=["1", "2"],
+            default_host="pubsublite.googleapis.com",
+            ssl_credentials=None,
+            options=[
+                ("grpc.max_send_message_length", -1),
+                ("grpc.max_receive_message_length", -1),
+            ],
+        )
+
+
+@pytest.mark.parametrize(
+    "transport_class,grpc_helpers",
+    [
+        (transports.CursorServiceGrpcTransport, grpc_helpers),
+        (transports.CursorServiceGrpcAsyncIOTransport, grpc_helpers_async),
+    ],
+)
+@requires_api_core_lt_1_26_0
+def test_cursor_service_transport_create_channel_old_api_core(
+    transport_class, grpc_helpers
+):
+    # If credentials and host are not provided, the transport class should use
+    # ADC credentials.
+    with mock.patch.object(
+        google.auth, "default", autospec=True
+    ) as adc, mock.patch.object(
+        grpc_helpers, "create_channel", autospec=True
+    ) as create_channel:
+        creds = ga_credentials.AnonymousCredentials()
+        adc.return_value = (creds, None)
+        transport_class(quota_project_id="octopus")
+
+        create_channel.assert_called_with(
+            "pubsublite.googleapis.com:443",
+            credentials=creds,
+            credentials_file=None,
+            quota_project_id="octopus",
+            scopes=("https://www.googleapis.com/auth/cloud-platform",),
+            ssl_credentials=None,
+            options=[
+                ("grpc.max_send_message_length", -1),
+                ("grpc.max_receive_message_length", -1),
+            ],
+        )
+
+
+@pytest.mark.parametrize(
+    "transport_class,grpc_helpers",
+    [
+        (transports.CursorServiceGrpcTransport, grpc_helpers),
+        (transports.CursorServiceGrpcAsyncIOTransport, grpc_helpers_async),
+    ],
+)
+@requires_api_core_lt_1_26_0
+def test_cursor_service_transport_create_channel_user_scopes(
+    transport_class, grpc_helpers
+):
+    # If credentials and host are not provided, the transport class should use
+    # ADC credentials.
+    with mock.patch.object(
+        google.auth, "default", autospec=True
+    ) as adc, mock.patch.object(
+        grpc_helpers, "create_channel", autospec=True
+    ) as create_channel:
+        creds = ga_credentials.AnonymousCredentials()
+        adc.return_value = (creds, None)
+
+        transport_class(quota_project_id="octopus", scopes=["1", "2"])
+
+        create_channel.assert_called_with(
+            "pubsublite.googleapis.com:443",
+            credentials=creds,
+            credentials_file=None,
+            quota_project_id="octopus",
+            scopes=["1", "2"],
+            ssl_credentials=None,
+            options=[
+                ("grpc.max_send_message_length", -1),
+                ("grpc.max_receive_message_length", -1),
+            ],
+        )
+
+
 @pytest.mark.parametrize(
     "transport_class",
     [
@@ -1155,7 +1396,7 @@ def test_cursor_service_transport_auth_adc():
     ],
 )
 def test_cursor_service_grpc_transport_client_cert_source_for_mtls(transport_class):
-    cred = credentials.AnonymousCredentials()
+    cred = ga_credentials.AnonymousCredentials()
 
     # Check ssl_channel_credentials is used if provided.
     with mock.patch.object(transport_class, "create_channel") as mock_create_channel:
@@ -1194,7 +1435,7 @@ def test_cursor_service_grpc_transport_client_cert_source_for_mtls(transport_cla
 
 def test_cursor_service_host_no_port():
     client = CursorServiceClient(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="pubsublite.googleapis.com"
         ),
@@ -1204,7 +1445,7 @@ def test_cursor_service_host_no_port():
 
 def test_cursor_service_host_with_port():
     client = CursorServiceClient(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="pubsublite.googleapis.com:8000"
         ),
@@ -1258,9 +1499,9 @@ def test_cursor_service_transport_channel_mtls_with_client_cert_source(transport
             mock_grpc_channel = mock.Mock()
             grpc_create_channel.return_value = mock_grpc_channel
 
-            cred = credentials.AnonymousCredentials()
+            cred = ga_credentials.AnonymousCredentials()
             with pytest.warns(DeprecationWarning):
-                with mock.patch.object(auth, "default") as adc:
+                with mock.patch.object(google.auth, "default") as adc:
                     adc.return_value = (cred, None)
                     transport = transport_class(
                         host="squid.clam.whelk",
@@ -1338,7 +1579,6 @@ def test_subscription_path():
     project = "squid"
     location = "clam"
     subscription = "whelk"
-
     expected = "projects/{project}/locations/{location}/subscriptions/{subscription}".format(
         project=project, location=location, subscription=subscription,
     )
@@ -1361,7 +1601,6 @@ def test_parse_subscription_path():
 
 def test_common_billing_account_path():
     billing_account = "cuttlefish"
-
     expected = "billingAccounts/{billing_account}".format(
         billing_account=billing_account,
     )
@@ -1382,7 +1621,6 @@ def test_parse_common_billing_account_path():
 
 def test_common_folder_path():
     folder = "winkle"
-
     expected = "folders/{folder}".format(folder=folder,)
     actual = CursorServiceClient.common_folder_path(folder)
     assert expected == actual
@@ -1401,7 +1639,6 @@ def test_parse_common_folder_path():
 
 def test_common_organization_path():
     organization = "scallop"
-
     expected = "organizations/{organization}".format(organization=organization,)
     actual = CursorServiceClient.common_organization_path(organization)
     assert expected == actual
@@ -1420,7 +1657,6 @@ def test_parse_common_organization_path():
 
 def test_common_project_path():
     project = "squid"
-
     expected = "projects/{project}".format(project=project,)
     actual = CursorServiceClient.common_project_path(project)
     assert expected == actual
@@ -1440,7 +1676,6 @@ def test_parse_common_project_path():
 def test_common_location_path():
     project = "whelk"
     location = "octopus"
-
     expected = "projects/{project}/locations/{location}".format(
         project=project, location=location,
     )
@@ -1467,7 +1702,7 @@ def test_client_withDEFAULT_CLIENT_INFO():
         transports.CursorServiceTransport, "_prep_wrapped_messages"
     ) as prep:
         client = CursorServiceClient(
-            credentials=credentials.AnonymousCredentials(), client_info=client_info,
+            credentials=ga_credentials.AnonymousCredentials(), client_info=client_info,
         )
         prep.assert_called_once_with(client_info)
 
@@ -1476,6 +1711,6 @@ def test_client_withDEFAULT_CLIENT_INFO():
     ) as prep:
         transport_class = CursorServiceClient.get_transport_class()
         transport = transport_class(
-            credentials=credentials.AnonymousCredentials(), client_info=client_info,
+            credentials=ga_credentials.AnonymousCredentials(), client_info=client_info,
         )
         prep.assert_called_once_with(client_info)
