@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,9 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import os
 import mock
+import packaging.version
 
 import grpc
 from grpc.experimental import aio
@@ -24,13 +23,13 @@ import math
 import pytest
 from proto.marshal.rules.dates import DurationRule, TimestampRule
 
-from google import auth
+
 from google.api_core import client_options
-from google.api_core import exceptions
+from google.api_core import exceptions as core_exceptions
 from google.api_core import gapic_v1
 from google.api_core import grpc_helpers
 from google.api_core import grpc_helpers_async
-from google.auth import credentials
+from google.auth import credentials as ga_credentials
 from google.auth.exceptions import MutualTLSChannelError
 from google.cloud.pubsublite_v1.services.topic_stats_service import (
     TopicStatsServiceAsyncClient,
@@ -39,10 +38,40 @@ from google.cloud.pubsublite_v1.services.topic_stats_service import (
     TopicStatsServiceClient,
 )
 from google.cloud.pubsublite_v1.services.topic_stats_service import transports
+from google.cloud.pubsublite_v1.services.topic_stats_service.transports.base import (
+    _API_CORE_VERSION,
+)
+from google.cloud.pubsublite_v1.services.topic_stats_service.transports.base import (
+    _GOOGLE_AUTH_VERSION,
+)
 from google.cloud.pubsublite_v1.types import common
 from google.cloud.pubsublite_v1.types import topic_stats
 from google.oauth2 import service_account
-from google.protobuf import timestamp_pb2 as timestamp  # type: ignore
+from google.protobuf import timestamp_pb2  # type: ignore
+import google.auth
+
+
+# TODO(busunkim): Once google-api-core >= 1.26.0 is required:
+# - Delete all the api-core and auth "less than" test cases
+# - Delete these pytest markers (Make the "greater than or equal to" tests the default).
+requires_google_auth_lt_1_25_0 = pytest.mark.skipif(
+    packaging.version.parse(_GOOGLE_AUTH_VERSION) >= packaging.version.parse("1.25.0"),
+    reason="This test requires google-auth < 1.25.0",
+)
+requires_google_auth_gte_1_25_0 = pytest.mark.skipif(
+    packaging.version.parse(_GOOGLE_AUTH_VERSION) < packaging.version.parse("1.25.0"),
+    reason="This test requires google-auth >= 1.25.0",
+)
+
+requires_api_core_lt_1_26_0 = pytest.mark.skipif(
+    packaging.version.parse(_API_CORE_VERSION) >= packaging.version.parse("1.26.0"),
+    reason="This test requires google-api-core < 1.26.0",
+)
+
+requires_api_core_gte_1_26_0 = pytest.mark.skipif(
+    packaging.version.parse(_API_CORE_VERSION) < packaging.version.parse("1.26.0"),
+    reason="This test requires google-api-core >= 1.26.0",
+)
 
 
 def client_cert_source_callback():
@@ -94,7 +123,7 @@ def test__get_default_mtls_endpoint():
     "client_class", [TopicStatsServiceClient, TopicStatsServiceAsyncClient,]
 )
 def test_topic_stats_service_client_from_service_account_info(client_class):
-    creds = credentials.AnonymousCredentials()
+    creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_info"
     ) as factory:
@@ -111,7 +140,7 @@ def test_topic_stats_service_client_from_service_account_info(client_class):
     "client_class", [TopicStatsServiceClient, TopicStatsServiceAsyncClient,]
 )
 def test_topic_stats_service_client_from_service_account_file(client_class):
-    creds = credentials.AnonymousCredentials()
+    creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_file"
     ) as factory:
@@ -164,7 +193,7 @@ def test_topic_stats_service_client_client_options(
 ):
     # Check that if channel is provided we won't create a new one.
     with mock.patch.object(TopicStatsServiceClient, "get_transport_class") as gtc:
-        transport = transport_class(credentials=credentials.AnonymousCredentials())
+        transport = transport_class(credentials=ga_credentials.AnonymousCredentials())
         client = client_class(transport=transport)
         gtc.assert_not_called()
 
@@ -462,7 +491,7 @@ def test_compute_message_stats(
     transport: str = "grpc", request_type=topic_stats.ComputeMessageStatsRequest
 ):
     client = TopicStatsServiceClient(
-        credentials=credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -477,21 +506,16 @@ def test_compute_message_stats(
         call.return_value = topic_stats.ComputeMessageStatsResponse(
             message_count=1389, message_bytes=1387,
         )
-
         response = client.compute_message_stats(request)
 
         # Establish that the underlying gRPC stub method was called.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == topic_stats.ComputeMessageStatsRequest()
 
     # Establish that the response is the type that we expect.
-
     assert isinstance(response, topic_stats.ComputeMessageStatsResponse)
-
     assert response.message_count == 1389
-
     assert response.message_bytes == 1387
 
 
@@ -503,7 +527,7 @@ def test_compute_message_stats_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
     client = TopicStatsServiceClient(
-        credentials=credentials.AnonymousCredentials(), transport="grpc",
+        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
     )
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -513,7 +537,6 @@ def test_compute_message_stats_empty_call():
         client.compute_message_stats()
         call.assert_called()
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == topic_stats.ComputeMessageStatsRequest()
 
 
@@ -522,7 +545,7 @@ async def test_compute_message_stats_async(
     transport: str = "grpc_asyncio", request_type=topic_stats.ComputeMessageStatsRequest
 ):
     client = TopicStatsServiceAsyncClient(
-        credentials=credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -539,20 +562,16 @@ async def test_compute_message_stats_async(
                 message_count=1389, message_bytes=1387,
             )
         )
-
         response = await client.compute_message_stats(request)
 
         # Establish that the underlying gRPC stub method was called.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == topic_stats.ComputeMessageStatsRequest()
 
     # Establish that the response is the type that we expect.
     assert isinstance(response, topic_stats.ComputeMessageStatsResponse)
-
     assert response.message_count == 1389
-
     assert response.message_bytes == 1387
 
 
@@ -562,11 +581,12 @@ async def test_compute_message_stats_async_from_dict():
 
 
 def test_compute_message_stats_field_headers():
-    client = TopicStatsServiceClient(credentials=credentials.AnonymousCredentials(),)
+    client = TopicStatsServiceClient(credentials=ga_credentials.AnonymousCredentials(),)
 
     # Any value that is part of the HTTP/1.1 URI should be sent as
     # a field header. Set these to a non-empty value.
     request = topic_stats.ComputeMessageStatsRequest()
+
     request.topic = "topic/value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -574,7 +594,6 @@ def test_compute_message_stats_field_headers():
         type(client.transport.compute_message_stats), "__call__"
     ) as call:
         call.return_value = topic_stats.ComputeMessageStatsResponse()
-
         client.compute_message_stats(request)
 
         # Establish that the underlying gRPC stub method was called.
@@ -590,12 +609,13 @@ def test_compute_message_stats_field_headers():
 @pytest.mark.asyncio
 async def test_compute_message_stats_field_headers_async():
     client = TopicStatsServiceAsyncClient(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
 
     # Any value that is part of the HTTP/1.1 URI should be sent as
     # a field header. Set these to a non-empty value.
     request = topic_stats.ComputeMessageStatsRequest()
+
     request.topic = "topic/value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -605,7 +625,6 @@ async def test_compute_message_stats_field_headers_async():
         call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
             topic_stats.ComputeMessageStatsResponse()
         )
-
         await client.compute_message_stats(request)
 
         # Establish that the underlying gRPC stub method was called.
@@ -622,7 +641,7 @@ def test_compute_head_cursor(
     transport: str = "grpc", request_type=topic_stats.ComputeHeadCursorRequest
 ):
     client = TopicStatsServiceClient(
-        credentials=credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -635,17 +654,14 @@ def test_compute_head_cursor(
     ) as call:
         # Designate an appropriate return value for the call.
         call.return_value = topic_stats.ComputeHeadCursorResponse()
-
         response = client.compute_head_cursor(request)
 
         # Establish that the underlying gRPC stub method was called.
         assert len(call.mock_calls) == 1
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == topic_stats.ComputeHeadCursorRequest()
 
     # Establish that the response is the type that we expect.
-
     assert isinstance(response, topic_stats.ComputeHeadCursorResponse)
 
 
@@ -657,7 +673,7 @@ def test_compute_head_cursor_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
     client = TopicStatsServiceClient(
-        credentials=credentials.AnonymousCredentials(), transport="grpc",
+        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
     )
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -667,7 +683,6 @@ def test_compute_head_cursor_empty_call():
         client.compute_head_cursor()
         call.assert_called()
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == topic_stats.ComputeHeadCursorRequest()
 
 
@@ -676,7 +691,7 @@ async def test_compute_head_cursor_async(
     transport: str = "grpc_asyncio", request_type=topic_stats.ComputeHeadCursorRequest
 ):
     client = TopicStatsServiceAsyncClient(
-        credentials=credentials.AnonymousCredentials(), transport=transport,
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
 
     # Everything is optional in proto3 as far as the runtime is concerned,
@@ -691,13 +706,11 @@ async def test_compute_head_cursor_async(
         call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
             topic_stats.ComputeHeadCursorResponse()
         )
-
         response = await client.compute_head_cursor(request)
 
         # Establish that the underlying gRPC stub method was called.
         assert len(call.mock_calls)
         _, args, _ = call.mock_calls[0]
-
         assert args[0] == topic_stats.ComputeHeadCursorRequest()
 
     # Establish that the response is the type that we expect.
@@ -710,11 +723,12 @@ async def test_compute_head_cursor_async_from_dict():
 
 
 def test_compute_head_cursor_field_headers():
-    client = TopicStatsServiceClient(credentials=credentials.AnonymousCredentials(),)
+    client = TopicStatsServiceClient(credentials=ga_credentials.AnonymousCredentials(),)
 
     # Any value that is part of the HTTP/1.1 URI should be sent as
     # a field header. Set these to a non-empty value.
     request = topic_stats.ComputeHeadCursorRequest()
+
     request.topic = "topic/value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -722,7 +736,6 @@ def test_compute_head_cursor_field_headers():
         type(client.transport.compute_head_cursor), "__call__"
     ) as call:
         call.return_value = topic_stats.ComputeHeadCursorResponse()
-
         client.compute_head_cursor(request)
 
         # Establish that the underlying gRPC stub method was called.
@@ -738,12 +751,13 @@ def test_compute_head_cursor_field_headers():
 @pytest.mark.asyncio
 async def test_compute_head_cursor_field_headers_async():
     client = TopicStatsServiceAsyncClient(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
 
     # Any value that is part of the HTTP/1.1 URI should be sent as
     # a field header. Set these to a non-empty value.
     request = topic_stats.ComputeHeadCursorRequest()
+
     request.topic = "topic/value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
@@ -753,7 +767,6 @@ async def test_compute_head_cursor_field_headers_async():
         call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
             topic_stats.ComputeHeadCursorResponse()
         )
-
         await client.compute_head_cursor(request)
 
         # Establish that the underlying gRPC stub method was called.
@@ -766,19 +779,161 @@ async def test_compute_head_cursor_field_headers_async():
     assert ("x-goog-request-params", "topic=topic/value",) in kw["metadata"]
 
 
+def test_compute_time_cursor(
+    transport: str = "grpc", request_type=topic_stats.ComputeTimeCursorRequest
+):
+    client = TopicStatsServiceClient(
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
+    )
+
+    # Everything is optional in proto3 as far as the runtime is concerned,
+    # and we are mocking out the actual API, so just send an empty request.
+    request = request_type()
+
+    # Mock the actual call within the gRPC stub, and fake the request.
+    with mock.patch.object(
+        type(client.transport.compute_time_cursor), "__call__"
+    ) as call:
+        # Designate an appropriate return value for the call.
+        call.return_value = topic_stats.ComputeTimeCursorResponse()
+        response = client.compute_time_cursor(request)
+
+        # Establish that the underlying gRPC stub method was called.
+        assert len(call.mock_calls) == 1
+        _, args, _ = call.mock_calls[0]
+        assert args[0] == topic_stats.ComputeTimeCursorRequest()
+
+    # Establish that the response is the type that we expect.
+    assert isinstance(response, topic_stats.ComputeTimeCursorResponse)
+
+
+def test_compute_time_cursor_from_dict():
+    test_compute_time_cursor(request_type=dict)
+
+
+def test_compute_time_cursor_empty_call():
+    # This test is a coverage failsafe to make sure that totally empty calls,
+    # i.e. request == None and no flattened fields passed, work.
+    client = TopicStatsServiceClient(
+        credentials=ga_credentials.AnonymousCredentials(), transport="grpc",
+    )
+
+    # Mock the actual call within the gRPC stub, and fake the request.
+    with mock.patch.object(
+        type(client.transport.compute_time_cursor), "__call__"
+    ) as call:
+        client.compute_time_cursor()
+        call.assert_called()
+        _, args, _ = call.mock_calls[0]
+        assert args[0] == topic_stats.ComputeTimeCursorRequest()
+
+
+@pytest.mark.asyncio
+async def test_compute_time_cursor_async(
+    transport: str = "grpc_asyncio", request_type=topic_stats.ComputeTimeCursorRequest
+):
+    client = TopicStatsServiceAsyncClient(
+        credentials=ga_credentials.AnonymousCredentials(), transport=transport,
+    )
+
+    # Everything is optional in proto3 as far as the runtime is concerned,
+    # and we are mocking out the actual API, so just send an empty request.
+    request = request_type()
+
+    # Mock the actual call within the gRPC stub, and fake the request.
+    with mock.patch.object(
+        type(client.transport.compute_time_cursor), "__call__"
+    ) as call:
+        # Designate an appropriate return value for the call.
+        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
+            topic_stats.ComputeTimeCursorResponse()
+        )
+        response = await client.compute_time_cursor(request)
+
+        # Establish that the underlying gRPC stub method was called.
+        assert len(call.mock_calls)
+        _, args, _ = call.mock_calls[0]
+        assert args[0] == topic_stats.ComputeTimeCursorRequest()
+
+    # Establish that the response is the type that we expect.
+    assert isinstance(response, topic_stats.ComputeTimeCursorResponse)
+
+
+@pytest.mark.asyncio
+async def test_compute_time_cursor_async_from_dict():
+    await test_compute_time_cursor_async(request_type=dict)
+
+
+def test_compute_time_cursor_field_headers():
+    client = TopicStatsServiceClient(credentials=ga_credentials.AnonymousCredentials(),)
+
+    # Any value that is part of the HTTP/1.1 URI should be sent as
+    # a field header. Set these to a non-empty value.
+    request = topic_stats.ComputeTimeCursorRequest()
+
+    request.topic = "topic/value"
+
+    # Mock the actual call within the gRPC stub, and fake the request.
+    with mock.patch.object(
+        type(client.transport.compute_time_cursor), "__call__"
+    ) as call:
+        call.return_value = topic_stats.ComputeTimeCursorResponse()
+        client.compute_time_cursor(request)
+
+        # Establish that the underlying gRPC stub method was called.
+        assert len(call.mock_calls) == 1
+        _, args, _ = call.mock_calls[0]
+        assert args[0] == request
+
+    # Establish that the field header was sent.
+    _, _, kw = call.mock_calls[0]
+    assert ("x-goog-request-params", "topic=topic/value",) in kw["metadata"]
+
+
+@pytest.mark.asyncio
+async def test_compute_time_cursor_field_headers_async():
+    client = TopicStatsServiceAsyncClient(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+
+    # Any value that is part of the HTTP/1.1 URI should be sent as
+    # a field header. Set these to a non-empty value.
+    request = topic_stats.ComputeTimeCursorRequest()
+
+    request.topic = "topic/value"
+
+    # Mock the actual call within the gRPC stub, and fake the request.
+    with mock.patch.object(
+        type(client.transport.compute_time_cursor), "__call__"
+    ) as call:
+        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
+            topic_stats.ComputeTimeCursorResponse()
+        )
+        await client.compute_time_cursor(request)
+
+        # Establish that the underlying gRPC stub method was called.
+        assert len(call.mock_calls)
+        _, args, _ = call.mock_calls[0]
+        assert args[0] == request
+
+    # Establish that the field header was sent.
+    _, _, kw = call.mock_calls[0]
+    assert ("x-goog-request-params", "topic=topic/value",) in kw["metadata"]
+
+
 def test_credentials_transport_error():
     # It is an error to provide credentials and a transport instance.
     transport = transports.TopicStatsServiceGrpcTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     with pytest.raises(ValueError):
         client = TopicStatsServiceClient(
-            credentials=credentials.AnonymousCredentials(), transport=transport,
+            credentials=ga_credentials.AnonymousCredentials(), transport=transport,
         )
 
     # It is an error to provide a credentials file and a transport instance.
     transport = transports.TopicStatsServiceGrpcTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     with pytest.raises(ValueError):
         client = TopicStatsServiceClient(
@@ -788,7 +943,7 @@ def test_credentials_transport_error():
 
     # It is an error to provide scopes and a transport instance.
     transport = transports.TopicStatsServiceGrpcTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     with pytest.raises(ValueError):
         client = TopicStatsServiceClient(
@@ -799,7 +954,7 @@ def test_credentials_transport_error():
 def test_transport_instance():
     # A client may be instantiated with a custom transport instance.
     transport = transports.TopicStatsServiceGrpcTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     client = TopicStatsServiceClient(transport=transport)
     assert client.transport is transport
@@ -808,13 +963,13 @@ def test_transport_instance():
 def test_transport_get_channel():
     # A client may be instantiated with a custom transport instance.
     transport = transports.TopicStatsServiceGrpcTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     channel = transport.grpc_channel
     assert channel
 
     transport = transports.TopicStatsServiceGrpcAsyncIOTransport(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
     )
     channel = transport.grpc_channel
     assert channel
@@ -829,23 +984,23 @@ def test_transport_get_channel():
 )
 def test_transport_adc(transport_class):
     # Test default credentials are used if not provided.
-    with mock.patch.object(auth, "default") as adc:
-        adc.return_value = (credentials.AnonymousCredentials(), None)
+    with mock.patch.object(google.auth, "default") as adc:
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
         transport_class()
         adc.assert_called_once()
 
 
 def test_transport_grpc_default():
     # A client should use the gRPC transport by default.
-    client = TopicStatsServiceClient(credentials=credentials.AnonymousCredentials(),)
+    client = TopicStatsServiceClient(credentials=ga_credentials.AnonymousCredentials(),)
     assert isinstance(client.transport, transports.TopicStatsServiceGrpcTransport,)
 
 
 def test_topic_stats_service_base_transport_error():
     # Passing both a credentials object and credentials_file should raise an error
-    with pytest.raises(exceptions.DuplicateCredentialArgs):
+    with pytest.raises(core_exceptions.DuplicateCredentialArgs):
         transport = transports.TopicStatsServiceTransport(
-            credentials=credentials.AnonymousCredentials(),
+            credentials=ga_credentials.AnonymousCredentials(),
             credentials_file="credentials.json",
         )
 
@@ -857,7 +1012,7 @@ def test_topic_stats_service_base_transport():
     ) as Transport:
         Transport.return_value = None
         transport = transports.TopicStatsServiceTransport(
-            credentials=credentials.AnonymousCredentials(),
+            credentials=ga_credentials.AnonymousCredentials(),
         )
 
     # Every method on the transport should just blindly
@@ -865,21 +1020,44 @@ def test_topic_stats_service_base_transport():
     methods = (
         "compute_message_stats",
         "compute_head_cursor",
+        "compute_time_cursor",
     )
     for method in methods:
         with pytest.raises(NotImplementedError):
             getattr(transport, method)(request=object())
 
 
+@requires_google_auth_gte_1_25_0
 def test_topic_stats_service_base_transport_with_credentials_file():
     # Instantiate the base transport with a credentials file
     with mock.patch.object(
-        auth, "load_credentials_from_file"
+        google.auth, "load_credentials_from_file", autospec=True
     ) as load_creds, mock.patch(
         "google.cloud.pubsublite_v1.services.topic_stats_service.transports.TopicStatsServiceTransport._prep_wrapped_messages"
     ) as Transport:
         Transport.return_value = None
-        load_creds.return_value = (credentials.AnonymousCredentials(), None)
+        load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
+        transport = transports.TopicStatsServiceTransport(
+            credentials_file="credentials.json", quota_project_id="octopus",
+        )
+        load_creds.assert_called_once_with(
+            "credentials.json",
+            scopes=None,
+            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
+            quota_project_id="octopus",
+        )
+
+
+@requires_google_auth_lt_1_25_0
+def test_topic_stats_service_base_transport_with_credentials_file_old_google_auth():
+    # Instantiate the base transport with a credentials file
+    with mock.patch.object(
+        google.auth, "load_credentials_from_file", autospec=True
+    ) as load_creds, mock.patch(
+        "google.cloud.pubsublite_v1.services.topic_stats_service.transports.TopicStatsServiceTransport._prep_wrapped_messages"
+    ) as Transport:
+        Transport.return_value = None
+        load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
         transport = transports.TopicStatsServiceTransport(
             credentials_file="credentials.json", quota_project_id="octopus",
         )
@@ -892,19 +1070,33 @@ def test_topic_stats_service_base_transport_with_credentials_file():
 
 def test_topic_stats_service_base_transport_with_adc():
     # Test the default credentials are used if credentials and credentials_file are None.
-    with mock.patch.object(auth, "default") as adc, mock.patch(
+    with mock.patch.object(google.auth, "default", autospec=True) as adc, mock.patch(
         "google.cloud.pubsublite_v1.services.topic_stats_service.transports.TopicStatsServiceTransport._prep_wrapped_messages"
     ) as Transport:
         Transport.return_value = None
-        adc.return_value = (credentials.AnonymousCredentials(), None)
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
         transport = transports.TopicStatsServiceTransport()
         adc.assert_called_once()
 
 
+@requires_google_auth_gte_1_25_0
 def test_topic_stats_service_auth_adc():
     # If no credentials are provided, we should use ADC credentials.
-    with mock.patch.object(auth, "default") as adc:
-        adc.return_value = (credentials.AnonymousCredentials(), None)
+    with mock.patch.object(google.auth, "default", autospec=True) as adc:
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
+        TopicStatsServiceClient()
+        adc.assert_called_once_with(
+            scopes=None,
+            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
+            quota_project_id=None,
+        )
+
+
+@requires_google_auth_lt_1_25_0
+def test_topic_stats_service_auth_adc_old_google_auth():
+    # If no credentials are provided, we should use ADC credentials.
+    with mock.patch.object(google.auth, "default", autospec=True) as adc:
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
         TopicStatsServiceClient()
         adc.assert_called_once_with(
             scopes=("https://www.googleapis.com/auth/cloud-platform",),
@@ -912,20 +1104,156 @@ def test_topic_stats_service_auth_adc():
         )
 
 
-def test_topic_stats_service_transport_auth_adc():
+@pytest.mark.parametrize(
+    "transport_class",
+    [
+        transports.TopicStatsServiceGrpcTransport,
+        transports.TopicStatsServiceGrpcAsyncIOTransport,
+    ],
+)
+@requires_google_auth_gte_1_25_0
+def test_topic_stats_service_transport_auth_adc(transport_class):
     # If credentials and host are not provided, the transport class should use
     # ADC credentials.
-    with mock.patch.object(auth, "default") as adc:
-        adc.return_value = (credentials.AnonymousCredentials(), None)
-        transports.TopicStatsServiceGrpcTransport(
-            host="squid.clam.whelk", quota_project_id="octopus"
+    with mock.patch.object(google.auth, "default", autospec=True) as adc:
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
+        transport_class(quota_project_id="octopus", scopes=["1", "2"])
+        adc.assert_called_once_with(
+            scopes=["1", "2"],
+            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
+            quota_project_id="octopus",
         )
+
+
+@pytest.mark.parametrize(
+    "transport_class",
+    [
+        transports.TopicStatsServiceGrpcTransport,
+        transports.TopicStatsServiceGrpcAsyncIOTransport,
+    ],
+)
+@requires_google_auth_lt_1_25_0
+def test_topic_stats_service_transport_auth_adc_old_google_auth(transport_class):
+    # If credentials and host are not provided, the transport class should use
+    # ADC credentials.
+    with mock.patch.object(google.auth, "default", autospec=True) as adc:
+        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
+        transport_class(quota_project_id="octopus")
         adc.assert_called_once_with(
             scopes=("https://www.googleapis.com/auth/cloud-platform",),
             quota_project_id="octopus",
         )
 
 
+@pytest.mark.parametrize(
+    "transport_class,grpc_helpers",
+    [
+        (transports.TopicStatsServiceGrpcTransport, grpc_helpers),
+        (transports.TopicStatsServiceGrpcAsyncIOTransport, grpc_helpers_async),
+    ],
+)
+@requires_api_core_gte_1_26_0
+def test_topic_stats_service_transport_create_channel(transport_class, grpc_helpers):
+    # If credentials and host are not provided, the transport class should use
+    # ADC credentials.
+    with mock.patch.object(
+        google.auth, "default", autospec=True
+    ) as adc, mock.patch.object(
+        grpc_helpers, "create_channel", autospec=True
+    ) as create_channel:
+        creds = ga_credentials.AnonymousCredentials()
+        adc.return_value = (creds, None)
+        transport_class(quota_project_id="octopus", scopes=["1", "2"])
+
+        create_channel.assert_called_with(
+            "pubsublite.googleapis.com:443",
+            credentials=creds,
+            credentials_file=None,
+            quota_project_id="octopus",
+            default_scopes=("https://www.googleapis.com/auth/cloud-platform",),
+            scopes=["1", "2"],
+            default_host="pubsublite.googleapis.com",
+            ssl_credentials=None,
+            options=[
+                ("grpc.max_send_message_length", -1),
+                ("grpc.max_receive_message_length", -1),
+            ],
+        )
+
+
+@pytest.mark.parametrize(
+    "transport_class,grpc_helpers",
+    [
+        (transports.TopicStatsServiceGrpcTransport, grpc_helpers),
+        (transports.TopicStatsServiceGrpcAsyncIOTransport, grpc_helpers_async),
+    ],
+)
+@requires_api_core_lt_1_26_0
+def test_topic_stats_service_transport_create_channel_old_api_core(
+    transport_class, grpc_helpers
+):
+    # If credentials and host are not provided, the transport class should use
+    # ADC credentials.
+    with mock.patch.object(
+        google.auth, "default", autospec=True
+    ) as adc, mock.patch.object(
+        grpc_helpers, "create_channel", autospec=True
+    ) as create_channel:
+        creds = ga_credentials.AnonymousCredentials()
+        adc.return_value = (creds, None)
+        transport_class(quota_project_id="octopus")
+
+        create_channel.assert_called_with(
+            "pubsublite.googleapis.com:443",
+            credentials=creds,
+            credentials_file=None,
+            quota_project_id="octopus",
+            scopes=("https://www.googleapis.com/auth/cloud-platform",),
+            ssl_credentials=None,
+            options=[
+                ("grpc.max_send_message_length", -1),
+                ("grpc.max_receive_message_length", -1),
+            ],
+        )
+
+
+@pytest.mark.parametrize(
+    "transport_class,grpc_helpers",
+    [
+        (transports.TopicStatsServiceGrpcTransport, grpc_helpers),
+        (transports.TopicStatsServiceGrpcAsyncIOTransport, grpc_helpers_async),
+    ],
+)
+@requires_api_core_lt_1_26_0
+def test_topic_stats_service_transport_create_channel_user_scopes(
+    transport_class, grpc_helpers
+):
+    # If credentials and host are not provided, the transport class should use
+    # ADC credentials.
+    with mock.patch.object(
+        google.auth, "default", autospec=True
+    ) as adc, mock.patch.object(
+        grpc_helpers, "create_channel", autospec=True
+    ) as create_channel:
+        creds = ga_credentials.AnonymousCredentials()
+        adc.return_value = (creds, None)
+
+        transport_class(quota_project_id="octopus", scopes=["1", "2"])
+
+        create_channel.assert_called_with(
+            "pubsublite.googleapis.com:443",
+            credentials=creds,
+            credentials_file=None,
+            quota_project_id="octopus",
+            scopes=["1", "2"],
+            ssl_credentials=None,
+            options=[
+                ("grpc.max_send_message_length", -1),
+                ("grpc.max_receive_message_length", -1),
+            ],
+        )
+
+
 @pytest.mark.parametrize(
     "transport_class",
     [
@@ -936,7 +1264,7 @@ def test_topic_stats_service_transport_auth_adc():
 def test_topic_stats_service_grpc_transport_client_cert_source_for_mtls(
     transport_class,
 ):
-    cred = credentials.AnonymousCredentials()
+    cred = ga_credentials.AnonymousCredentials()
 
     # Check ssl_channel_credentials is used if provided.
     with mock.patch.object(transport_class, "create_channel") as mock_create_channel:
@@ -975,7 +1303,7 @@ def test_topic_stats_service_grpc_transport_client_cert_source_for_mtls(
 
 def test_topic_stats_service_host_no_port():
     client = TopicStatsServiceClient(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="pubsublite.googleapis.com"
         ),
@@ -985,7 +1313,7 @@ def test_topic_stats_service_host_no_port():
 
 def test_topic_stats_service_host_with_port():
     client = TopicStatsServiceClient(
-        credentials=credentials.AnonymousCredentials(),
+        credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="pubsublite.googleapis.com:8000"
         ),
@@ -1041,9 +1369,9 @@ def test_topic_stats_service_transport_channel_mtls_with_client_cert_source(
             mock_grpc_channel = mock.Mock()
             grpc_create_channel.return_value = mock_grpc_channel
 
-            cred = credentials.AnonymousCredentials()
+            cred = ga_credentials.AnonymousCredentials()
             with pytest.warns(DeprecationWarning):
-                with mock.patch.object(auth, "default") as adc:
+                with mock.patch.object(google.auth, "default") as adc:
                     adc.return_value = (cred, None)
                     transport = transport_class(
                         host="squid.clam.whelk",
@@ -1121,7 +1449,6 @@ def test_topic_path():
     project = "squid"
     location = "clam"
     topic = "whelk"
-
     expected = "projects/{project}/locations/{location}/topics/{topic}".format(
         project=project, location=location, topic=topic,
     )
@@ -1144,7 +1471,6 @@ def test_parse_topic_path():
 
 def test_common_billing_account_path():
     billing_account = "cuttlefish"
-
     expected = "billingAccounts/{billing_account}".format(
         billing_account=billing_account,
     )
@@ -1165,7 +1491,6 @@ def test_parse_common_billing_account_path():
 
 def test_common_folder_path():
     folder = "winkle"
-
     expected = "folders/{folder}".format(folder=folder,)
     actual = TopicStatsServiceClient.common_folder_path(folder)
     assert expected == actual
@@ -1184,7 +1509,6 @@ def test_parse_common_folder_path():
 
 def test_common_organization_path():
     organization = "scallop"
-
     expected = "organizations/{organization}".format(organization=organization,)
     actual = TopicStatsServiceClient.common_organization_path(organization)
     assert expected == actual
@@ -1203,7 +1527,6 @@ def test_parse_common_organization_path():
 
 def test_common_project_path():
     project = "squid"
-
     expected = "projects/{project}".format(project=project,)
     actual = TopicStatsServiceClient.common_project_path(project)
     assert expected == actual
@@ -1223,7 +1546,6 @@ def test_parse_common_project_path():
 def test_common_location_path():
     project = "whelk"
     location = "octopus"
-
     expected = "projects/{project}/locations/{location}".format(
         project=project, location=location,
     )
@@ -1250,7 +1572,7 @@ def test_client_withDEFAULT_CLIENT_INFO():
         transports.TopicStatsServiceTransport, "_prep_wrapped_messages"
     ) as prep:
         client = TopicStatsServiceClient(
-            credentials=credentials.AnonymousCredentials(), client_info=client_info,
+            credentials=ga_credentials.AnonymousCredentials(), client_info=client_info,
         )
         prep.assert_called_once_with(client_info)
 
@@ -1259,6 +1581,6 @@ def test_client_withDEFAULT_CLIENT_INFO():
     ) as prep:
         transport_class = TopicStatsServiceClient.get_transport_class()
         transport = transport_class(
-            credentials=credentials.AnonymousCredentials(), client_info=client_info,
+            credentials=ga_credentials.AnonymousCredentials(), client_info=client_info,
         )
         prep.assert_called_once_with(client_info)
